@@ -17,7 +17,7 @@
 #ifndef MONGOC_GRIDFS_FILE_LIST_PRIVATE_H
 #define MONGOC_GRIDFS_FILE_LIST_PRIVATE_H
 
-#if !defined (MONGOC_I_AM_A_DRIVER) && !defined (MONGOC_COMPILATION)
+#if !defined (MONGOC_COMPILATION)
 #error "Only <mongoc.h> can be included directly."
 #endif
 
@@ -39,9 +39,12 @@ struct _mongoc_gridfs_file_list_t
 };
 
 
-mongoc_gridfs_file_list_t *_mongoc_gridfs_file_list_new (mongoc_gridfs_t *gridfs,
-                                                         const bson_t    *query,
-                                                         uint32_t         limit);
+mongoc_gridfs_file_list_t *_mongoc_gridfs_file_list_new           (mongoc_gridfs_t *gridfs,
+                                                                   const bson_t    *query,
+                                                                   uint32_t         limit);
+mongoc_gridfs_file_list_t *_mongoc_gridfs_file_list_new_with_opts (mongoc_gridfs_t *gridfs,
+                                                                   const bson_t    *filter,
+                                                                   const bson_t    *opts);
 
 
 BSON_END_DECLS
