@@ -31,7 +31,8 @@
 #include "mongoc-gridfs-file-list.h"
 #include "mongoc-gridfs-file-list-private.h"
 #include "mongoc-client.h"
-#include "mongoc-trace.h"
+#include "mongoc-trace-private.h"
+#include "mongoc-cursor-private.h"
 
 #define MONGOC_GRIDFS_STREAM_CHUNK 4096
 
@@ -209,6 +210,39 @@ mongoc_gridfs_find_one (mongoc_gridfs_t *gridfs,
 }
 
 
+/** find all matching gridfs files */
+mongoc_gridfs_file_list_t *
+mongoc_gridfs_find_with_opts (mongoc_gridfs_t *gridfs,
+                              const bson_t    *filter,
+                              const bson_t    *opts)
+{
+   return _mongoc_gridfs_file_list_new_with_opts (gridfs, filter, opts);
+}
+
+
+/** find a single gridfs file */
+mongoc_gridfs_file_t *
+mongoc_gridfs_find_one_with_opts (mongoc_gridfs_t *gridfs,
+                                  const bson_t    *filter,
+                                  const bson_t    *opts,
+                                  bson_error_t    *error)
+{
+   mongoc_gridfs_file_list_t *list;
+   mongoc_gridfs_file_t *file;
+
+   ENTRY;
+
+   list = _mongoc_gridfs_file_list_new_with_opts (gridfs, filter, opts);
+
+   file = mongoc_gridfs_file_list_next (list);
+   mongoc_gridfs_file_list_error(list, error);
+
+   mongoc_gridfs_file_list_destroy (list);
+
+   RETURN (file);
+}
+
+
 /** find a single gridfs file by filename */
 mongoc_gridfs_file_t *
 mongoc_gridfs_find_one_by_filename (mongoc_gridfs_t *gridfs,
@@ -217,15 +251,15 @@ mongoc_gridfs_find_one_by_filename (mongoc_gridfs_t *gridfs,
 {
    mongoc_gridfs_file_t *file;
 
-   bson_t query;
+   bson_t filter;
 
-   bson_init (&query);
+   bson_init (&filter);
 
-   bson_append_utf8 (&query, "filename", -1, filename, -1);
+   bson_append_utf8 (&filter, "filename", -1, filename, -1);
 
-   file = mongoc_gridfs_find_one (gridfs, &query, error);
+   file = mongoc_gridfs_find_one_with_opts (gridfs, &filter, NULL, error);
 
-   bson_destroy (&query);
+   bson_destroy (&filter);
 
    return file;
 }
@@ -356,8 +390,11 @@ mongoc_gridfs_remove_by_filename (mongoc_gridfs_t *gridfs,
    BSON_APPEND_UTF8 (&q, "filename", filename);
    BSON_APPEND_INT32 (&fields, "_id", 1);
 
-   cursor = mongoc_collection_find (gridfs->files, MONGOC_QUERY_NONE, 0, 0, 0,
-                                    &q, &fields, NULL);
+   cursor = _mongoc_cursor_new (gridfs->client, gridfs->files->ns,
+                                MONGOC_QUERY_NONE, 0, 0, 0,
+                                false /* is command */,
+                                &q, &fields, NULL, NULL);
+
    BSON_ASSERT (cursor);
 
    while (mongoc_cursor_next (cursor, &doc)) {
