@@ -53,8 +53,17 @@ install_json_test_suite(TestSuite  *suite,
 mongoc_topology_description_type_t
 topology_type_from_test(const char *type);
 
-const char *
-topology_type_to_string(mongoc_topology_description_type_t type);
+mongoc_server_description_t *
+server_description_by_hostname (mongoc_topology_description_t *topology,
+                                const char *address);
+
+void
+process_sdam_test_ismaster_responses (bson_t                        *phase,
+                                      mongoc_topology_description_t *td);
+
+void
+check_json_apm_events (const bson_t *events,
+                       const bson_t *expectations);
 
 void
 test_server_selection_logic_cb (bson_t *test);
