@@ -69,11 +69,6 @@ const char *mock_server_get_host_and_port (mock_server_t *server);
 
 uint16_t mock_server_get_port (mock_server_t *server);
 
-bool mock_server_get_verbose (mock_server_t *server);
-
-void mock_server_set_verbose (mock_server_t *server,
-                              bool verbose);
-
 int64_t mock_server_get_request_timeout_msec (mock_server_t *server);
 
 void mock_server_set_request_timeout_msec (mock_server_t *server,
@@ -103,7 +98,7 @@ request_t *mock_server_receives_query (mock_server_t *server,
                                        const char *ns,
                                        mongoc_query_flags_t flags,
                                        uint32_t skip,
-                                       uint32_t n_return,
+                                       int32_t n_return,
                                        const char *query_json,
                                        const char *fields_json);
 
@@ -130,7 +125,7 @@ request_t * mock_server_receives_delete (mock_server_t *server,
 
 request_t *mock_server_receives_getmore (mock_server_t *server,
                                          const char *ns,
-                                         uint32_t n_return,
+                                         int32_t n_return,
                                          int64_t cursor_id);
 
 request_t *mock_server_receives_kill_cursors (mock_server_t *server,
