@@ -23,6 +23,7 @@
 #include "mongoc.h"
 
 #include "mongoc-rpc-private.h"
+#include "sync-queue.h"
 
 struct _mock_server_t;  /* forward declaration */
 
@@ -39,6 +40,7 @@ typedef struct _request_t
    char *command_name;
    char *as_str;
    mongoc_array_t docs;  /* array of bson_t pointers */
+   sync_queue_t *replies;
 } request_t;
 
 
@@ -46,7 +48,8 @@ request_t *request_new (const mongoc_buffer_t *buffer,
                         int32_t msg_len,
                         struct _mock_server_t *server,
                         mongoc_stream_t *client,
-                        uint16_t client_port);
+                        uint16_t client_port,
+                        sync_queue_t *replies);
 
 const bson_t * request_get_doc (const request_t *request,
                                 int n);
@@ -58,7 +61,7 @@ bool request_matches_query (const request_t *request,
                             const char *ns,
                             mongoc_query_flags_t flags,
                             uint32_t skip,
-                            uint32_t n_return,
+                            int32_t n_return,
                             const char *query_json,
                             const char *fields_json,
                             bool is_command);
@@ -86,7 +89,7 @@ bool request_matches_delete (const request_t *request,
 
 bool request_matches_getmore (const request_t *request,
                               const char *ns,
-                              uint32_t n_return,
+                              int32_t n_return,
                               int64_t cursor_id);
 
 bool request_matches_kill_cursors (const request_t *request,
