@@ -12,6 +12,102 @@
 #include "test-conveniences.h"
 
 
+static void
+test_create_with_write_concern (void)
+{
+   mongoc_database_t *database;
+   mongoc_collection_t *collection;
+   mongoc_client_t *client;
+   bson_error_t error = { 0 };
+   mongoc_write_concern_t *bad_wc;
+   mongoc_write_concern_t *good_wc;
+   bool wire_version_5;
+   bson_t *opts = NULL;
+   char *dbname;
+   char *name;
+
+   capture_logs (true);
+   opts = bson_new ();
+
+   client = test_framework_client_new ();
+   assert (client);
+   mongoc_client_set_error_api (client, 2);
+
+   bad_wc = mongoc_write_concern_new ();
+   good_wc = mongoc_write_concern_new ();
+
+   wire_version_5 = test_framework_max_wire_version_at_least (5);
+
+   dbname = gen_collection_name ("dbtest");
+   database = mongoc_client_get_database (client, dbname);
+   assert (database);
+
+   name = gen_collection_name ("create_collection");
+
+   /* writeConcern that will not pass mongoc_write_concern_is_valid */
+   bad_wc->wtimeout = -10;
+   bson_reinit (opts);
+   mongoc_write_concern_append_bad (bad_wc, opts);
+   collection = mongoc_database_create_collection (
+      database, name, opts, &error);
+   ASSERT_ERROR_CONTAINS (error, MONGOC_ERROR_COMMAND,
+                          MONGOC_ERROR_COMMAND_INVALID_ARG,
+                          "Invalid writeConcern");
+   ASSERT (!collection);
+   bad_wc->wtimeout = 0;
+   error.code = 0;
+   error.domain = 0;
+
+   /* valid writeConcern on all configs */
+   mongoc_write_concern_set_w (good_wc, 1);
+   bson_reinit (opts);
+   mongoc_write_concern_append (good_wc, opts);
+   collection = mongoc_database_create_collection (
+      database, name, opts, &error);
+   ASSERT_OR_PRINT (collection, error);
+   ASSERT (!error.code);
+   ASSERT (!error.domain);
+
+   ASSERT_OR_PRINT (mongoc_collection_drop (collection, &error),
+                    error);
+
+   /* writeConcern that results in writeConcernError */
+   bad_wc->wtimeout = 0;
+   mongoc_write_concern_set_w (bad_wc, 99);
+   if (!test_framework_is_mongos ()) { /* skip if sharded */
+      bson_reinit (opts);
+      mongoc_write_concern_append_bad (bad_wc, opts);
+      collection = mongoc_database_create_collection (
+         database, name, opts, &error);
+
+      if (wire_version_5) {
+         ASSERT (!collection);
+         if (test_framework_is_replset ()) { /* replica set */
+            ASSERT_ERROR_CONTAINS (error, MONGOC_ERROR_WRITE_CONCERN,
+                                   100, "Write Concern error:");
+         } else { /* standalone */
+            ASSERT_CMPINT (error.domain, ==, MONGOC_ERROR_SERVER);
+            ASSERT_CMPINT (error.code, ==, 2);
+         }
+      } else { /* if wire_version <= 4, no error */
+         ASSERT_OR_PRINT (collection, error);
+         ASSERT (!error.code);
+         ASSERT (!error.domain);
+         ASSERT_OR_PRINT (mongoc_collection_drop (collection, &error),
+                          error);
+      }
+   }
+
+   mongoc_database_destroy (database);
+   bson_free (name);
+   bson_free (dbname);
+   bson_destroy (opts);
+   mongoc_write_concern_destroy (good_wc);
+   mongoc_write_concern_destroy (bad_wc);
+   mongoc_client_destroy (client);
+}
+
+
 static void
 test_copy (void)
 {
@@ -263,24 +359,105 @@ test_db_command_read_prefs_pooled (void)
 static void
 test_drop (void)
 {
-   mongoc_database_t *database;
    mongoc_client_t *client;
+   mongoc_database_t *database;
+   mongoc_collection_t *collection;
    bson_error_t error = { 0 };
+   bson_t *opts = NULL;
    char *dbname;
+   mongoc_write_concern_t *good_wc;
+   mongoc_write_concern_t *bad_wc;
+   bool wire_version_5;
+   bool r;
 
+   opts = bson_new ();
    client = test_framework_client_new ();
    assert (client);
+   mongoc_client_set_error_api (client, 2);
+
+   bad_wc = mongoc_write_concern_new ();
+   good_wc = mongoc_write_concern_new ();
+   wire_version_5 = test_framework_max_wire_version_at_least (5);
 
    dbname = gen_collection_name ("db_drop_test");
    database = mongoc_client_get_database (client, dbname);
-   bson_free (dbname);
+
+   /* MongoDB 3.2+ must create at least one replicated database before
+    * dropDatabase will check writeConcern, see SERVER-25601 */
+   collection = mongoc_database_get_collection (database, "collection");
+   r = mongoc_collection_insert (collection, MONGOC_INSERT_NONE, tmp_bson ("{}"),
+                                 NULL, &error);
+
+   ASSERT_OR_PRINT (r, error);
 
    ASSERT_OR_PRINT (mongoc_database_drop (database, &error), error);
    assert (!error.domain);
    assert (!error.code);
 
    mongoc_database_destroy (database);
+
+   /* invalid writeConcern */
+   bad_wc->wtimeout = -10;
+   database = mongoc_client_get_database (client, dbname);
+
+   bson_reinit (opts);
+   mongoc_write_concern_append_bad (bad_wc, opts);
+   ASSERT (!mongoc_database_drop_with_opts (database,
+                                            opts,
+                                            &error));
+   ASSERT_ERROR_CONTAINS (error, MONGOC_ERROR_COMMAND,
+                          MONGOC_ERROR_COMMAND_INVALID_ARG,
+                          "Invalid writeConcern");
+   bad_wc->wtimeout = 0;
+   error.code = 0;
+   error.domain = 0;
+
+   /* valid writeConcern */
+   mongoc_write_concern_set_w (good_wc, 1);
+
+   bson_reinit (opts);
+   mongoc_write_concern_append (good_wc, opts);
+   ASSERT_OR_PRINT (mongoc_database_drop_with_opts (database,
+                                                    opts,
+                                                    &error),
+                    error);
+   assert (!error.code);
+   assert (!error.domain);
+
+   /* invalid writeConcern */
+   mongoc_write_concern_set_w (bad_wc, 99);
+   mongoc_database_destroy (database);
+
+   if (!test_framework_is_mongos ()) { /* skip if sharded */
+      database = mongoc_client_get_database (client, dbname);
+      bson_reinit (opts);
+      mongoc_write_concern_append_bad (bad_wc, opts);
+      r = mongoc_database_drop_with_opts (database,
+                                          opts,
+                                          &error);
+      if (wire_version_5) {
+         ASSERT (!r);
+         if (test_framework_is_replset ()) {
+            ASSERT_ERROR_CONTAINS (error, MONGOC_ERROR_WRITE_CONCERN,
+                                   100, "Write Concern error:");
+         } else { /* standalone */
+            ASSERT_CMPINT (error.domain, ==, MONGOC_ERROR_SERVER);
+            ASSERT_CMPINT (error.code, ==, 2);
+         }
+      } else { /* if wire_version <= 4, no error */
+         ASSERT_OR_PRINT (r, error);
+         ASSERT (!error.code);
+         ASSERT (!error.domain);
+         mongoc_database_destroy (database);
+      }
+   }
+
+   bson_free (dbname);
+   bson_destroy (opts);
+   mongoc_collection_destroy (collection);
    mongoc_client_destroy (client);
+   mongoc_write_concern_destroy (good_wc);
+   mongoc_write_concern_destroy (bad_wc);
 }
 
 
@@ -310,10 +487,9 @@ test_create_collection (void)
    BSON_APPEND_INT32 (&options, "size", 1234);
    BSON_APPEND_INT32 (&options, "max", 4567);
    BSON_APPEND_BOOL (&options, "capped", true);
-   BSON_APPEND_BOOL (&options, "autoIndexId", true);
 
-   BSON_APPEND_DOCUMENT_BEGIN(&options, "storage", &storage_opts);
-   BSON_APPEND_DOCUMENT_BEGIN(&storage_opts, "wiredtiger", &wt_opts);
+   BSON_APPEND_DOCUMENT_BEGIN(&options, "storageEngine", &storage_opts);
+   BSON_APPEND_DOCUMENT_BEGIN(&storage_opts, "wiredTiger", &wt_opts);
    BSON_APPEND_UTF8(&wt_opts, "configString", "block_compressor=zlib");
    bson_append_document_end(&storage_opts, &wt_opts);
    bson_append_document_end(&options, &storage_opts);
@@ -374,7 +550,6 @@ test_get_collection_info (void)
    BSON_APPEND_INT32 (&capped_options, "max", 1024);
 
    autoindexid_name = gen_collection_name ("autoindexid");
-   BSON_APPEND_BOOL (&autoindexid_options, "autoIndexId", false);
 
    noopts_name = gen_collection_name ("noopts");
 
@@ -646,6 +821,8 @@ test_get_default_database (void)
 void
 test_database_install (TestSuite *suite)
 {
+   TestSuite_AddLive (suite, "/Database/create_with_write_concern",
+                      test_create_with_write_concern);
    TestSuite_AddLive (suite, "/Database/copy", test_copy);
    TestSuite_AddLive (suite, "/Database/has_collection", test_has_collection);
    TestSuite_AddLive (suite, "/Database/command", test_command);
