@@ -328,7 +328,7 @@ void
 test_stream_tls_error_install (TestSuite *suite)
 {
    /* TLS stream doesn't detect hangup promptly on Solaris for some reason */
-#ifndef MONGOC_ENABLE_SSL_SECURE_CHANNEL
+#if !defined(MONGOC_ENABLE_SSL_SECURE_CHANNEL) && !defined(MONGOC_ENABLE_SSL_LIBRESSL)
 #if !defined(__sun) && !defined(__APPLE__)
    TestSuite_Add (suite, "/TLS/hangup", test_mongoc_tls_hangup);
 #endif
